#!/usr/bin/python3

import os
import time
import json
import argparse
import requests
from requests_toolbelt.multipart.encoder import MultipartEncoder

class MobSF:
    def __init__(self, host, api_key):
        self.host = str(host)
        self.headers = {}
        self.headers['X-Mobsf-Api-Key'] = api_key

    def upload(self, files):
        if os.path.exists(files):
            data = MultipartEncoder(fields={'file': (files, open(files, 'rb'), 'application/octet-stream')})
            self.headers['Content-Type'] = data.content_type
        else:
            print("File not found!")
            exit()

        try:
            print("[+] Uploading '{}' to MobSF".format(files))
            resp = requests.post(self.host + "/api/v1/upload", headers=self.headers, data=data)
            return resp
        except requests.exceptions.ConnectionError:
            print("[!] Error connecting to server, please try again")
            exit()

    def scan(self, scan_type, filename, filehash):
        data = { "scan_type" : scan_type, "file_name" : filename, "hash" : filehash }
        self.headers['Content-Type'] = "application/x-www-form-urlencoded"

        print("[+] Scanning '{}' file....".format(filename))
        resp = requests.post(self.host + "/api/v1/scan", headers=self.headers, data=data)

        return resp.json()

    def report_json(self, filehash, output):
        data = { "hash" : filehash }
        print("[+] Generating report in JSON format")
        resp = requests.post(self.host + "/api/v1/report_json", headers=self.headers, data=data)

        print("[+] The scan result is saved as '{}'".format(output))
        with open(output, 'w') as f:
            json.dump(resp.json(), f, indent=2)

        return resp

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='MobSF CI/CD integration')
    parser.add_argument('--host', help="MobSF host URL", required=False)
    parser.add_argument('--api_key', help="MobSF API Key", required=False)
    parser.add_argument('--file', help="Location of APK file", required=True)
    parser.add_argument('--output', help="The output file of scan result", required=True)
    args = parser.parse_args()

    if "MOBSF_HOST" in os.environ:
        host = os.getenv('MOBSF_HOST')
    else:
        host = args.host
    if "MOBSF_API_KEY" in os.environ:
        api_key = os.getenv('MOBSF_API_KEY')
    else:
        api_key = args.api_key
    files = args.file
    output = args.output

    mobsf = MobSF(host, api_key)
    upload = mobsf.upload(files)

    if upload.status_code == 200:
        # {"analyzer": "static_analyzer", "status": "success", "hash": "32de32d5e7ffb3c9119e003521afd47a", "scan_type": "apk", "file_name": "app-debug.apk"}
        upload_data = upload.json()
        print("[+] File(s) uploaded successfully")

        scan_type = upload_data["scan_type"]
        filename = upload_data["file_name"]
        filehash = upload_data["hash"]

        time.sleep(3)

        mobsf.scan(scan_type, filename, filehash)
        mobsf.report_json(filehash, output)
    else:
        print("[!] Something went wrong, please debug " + str(upload.status_code))